/**
 * Title:       Order queue receiver for ESE, within ASETS project (ASE Trading System)
 * ClassName:   ESEOrderReceiverNoReply.java
 * Created:     January 20, 2010
 * Author:      Claudiu Vinte
 * Department:  Economic Informatics, The Bucharest Academy of Economic Studies
 *
 * Description: Implements an Order queue receiver within OpenMQ broker by extending AbstractQueueReceiver class
 */

package asets.testapi;

import asets.api.*;
import javax.jms.*;
import java.util.concurrent.ConcurrentLinkedQueue;


public class ESEOrderReceiverNoReply extends OrderReceiverNoReply
{
    private ConcurrentLinkedQueue<Order> ordersToSend = null;
    private ConcurrentLinkedQueue<Execution> executionsToSend = null;

    public ESEOrderReceiverNoReply(String imqAddress, String queueName, String userID)
            throws JMSException, Exception
    {
        super(imqAddress, queueName, userID);

        ordersToSend = new ConcurrentLinkedQueue<Order>();
        executionsToSend = new ConcurrentLinkedQueue<Execution>();
    }

    // Receive messages from Order queue
    @Override
    public void onMessage(Message message)
    {
        try
        {
            if (message instanceof ObjectMessage)
            {
                if ( (message.getStringProperty("MSG_TYPE").equals(Enums.MessageType.ORDER.toString())) &&
                    (message.getStringProperty("USER_ID").equals("OMS-MOCK")))
                    // Client Orders are received by ESE either from OMS or from PROG
                    // Market Orders are received by OMS from ESE
                    // ESE      - Exchange Simulation Engine
                    // OMS      - Order Management Server
                    // PROG     - Pseudo Random Order Generator
                {
                    ObjectMessage objectMessage = (ObjectMessage)message;
                    processOrder((Order)objectMessage.getObject());
                }
            }
        } catch (JMSException e) {
            e.printStackTrace();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    protected void processOrder(Order order) throws Exception
    {
        System.out.println("ESE RECEIVED ORDER FROM OMS");
        System.out.println("User ID: " + order.getUserID());
        System.out.println("Order Type: " + order.getOrderType().toString());
        System.out.println("Symbol: " + order.getSymbol());
        System.out.println("Side: " + order.getOrderSide().toString());
        System.out.println("Quantity: " + order.getOrderQuant());
        System.out.println("Price: " + order.getOrderPrice());
        System.out.println("Order Status: " + order.getOrderStatus().toString());
        System.out.println("Market Status: " + order.getMarketStatus().toString());
        System.out.println();

        matchOrder(order);

        Order updatedOrder = updateOrder(order);
        ordersToSend.offer(updatedOrder);

        Execution execution = createExecution(order);
        executionsToSend.offer(execution);
    }

    public ConcurrentLinkedQueue<Order> getOrdersToSend()
    {
        return this.ordersToSend;
    }

    public ConcurrentLinkedQueue<Execution> getExecutionsToSend()
    {
        return this.executionsToSend;
    }

    protected void matchOrder(Order order)
    {
        order.setOrderStatus(Enums.OrderStatus.EXECUTED);
        order.setMarketStatus(Enums.MarketStatus.FULLY_EXECUTED);

        System.out.println("ESE ORDER FULLY EXECUTED");
        System.out.println("User ID: " + order.getUserID());
        System.out.println("Order Type: " + order.getOrderType().toString());
        System.out.println("Symbol: " + order.getSymbol());
        System.out.println("Side: " + order.getOrderSide().toString());
        System.out.println("Quantity: " + order.getOrderQuant());
        System.out.println("Price: " + order.getOrderPrice());
        System.out.println("Order Status: " + order.getOrderStatus().toString());
        System.out.println("Market Status: " + order.getMarketStatus().toString());
        System.out.println();
    }

    protected Execution createExecution(Order order)
    {
        Execution execution = new Execution();

        execution.setUserID(order.getUserID());
        execution.setSymbol(order.getSymbol());
        execution.setExecSide(order.getOrderSide());
        execution.setExecQuant(order.getOrderQuant());
        execution.setExecPrice(order.getOrderPrice());

        return execution;
    }

    protected Order updateOrder(Order order)
    {
        Order updatedOrder = new Order();

        updatedOrder.setUserID(order.getUserID());
        updatedOrder.setSymbol(order.getSymbol());
        updatedOrder.setOrderType(order.getOrderType());
        updatedOrder.setOrderSide(order.getOrderSide());
        updatedOrder.setOrderQuant(order.getOrderQuant());
        updatedOrder.setOrderPrice(order.getOrderPrice());
        updatedOrder.setOrderStatus(order.getOrderStatus());
        updatedOrder.setMarketStatus(order.getMarketStatus());

        return updatedOrder;
    }
}
